#!/usr/bin/env python
# -*- coding: utf-8 -*-

# system modules
import glob
import itertools
import os
import pickle
import pprint
import re
import sys

# custom modules
import mendeley_client as mc
import output_tracker as ot

# settings
group_to_sync 	= u'GROUPNAME'

# output handling
output = ot.output_tracker()

# data structures and helper
class paper(object):
	def __init__(self, id, timestamp, citation_key, doi):
		self._id = id
		self._timestamp = timestamp
		self._citation = citation_key
		self._doi = doi

def bibtex_escape(entry):
	s = [u'ä', u'ö', u'ü']
	r = [r'{\"a}', r'{\"o}', r'{\"u}']
	for p in zip(s, r):
		entry = entry.replace(*p)
	return entry

# extract citation keys from .tex file
if len(sys.argv) == 1:
	files = glob.glob('./*.tex')
else:
	files = sys.argv[1:]
tex_keys = set()
output.print_info('Parsing LaTeX files for \cite{} commands').add_level()
for fn in files:
	oldlen = len(tex_keys)
	output.print_info('Parsing file %s' % fn)
	fh = open(fn, 'r')
	matches = [re.findall(r'\cite{([^}]*)}', _) for _ in fh.readlines() if not _.strip().startswith('%')]
	groups = [_ for _ in matches if _ != []]
	for group in itertools.chain(*groups):
		keys = group.split(',')
		tex_keys.update([_.strip() for _ in keys])
	output.add_level().print_info('Found %d additional entries.' % (len(tex_keys) - oldlen)).del_level()
output.print_info('Found %d citation keys in total.' % len(tex_keys)).del_level()

# check exported citations
fhbib = open('mendeley-export.bib', 'a+')
matches = [re.findall(r'@article{([^,]*),', _, re.IGNORECASE) for _ in fhbib.readlines()]
groups = [_ for _ in matches if _ != []]
bib_keys = set(itertools.chain(*groups))

# difference
diff = tex_keys - bib_keys
if len(diff) == 0:
	output.print_info('All citation keys are in the bibfile.')
else:
	output.print_info('%d missing citation key(s)' % len(diff))

# connection
basepath = os.path.dirname(os.path.realpath(__file__))
mendeley = mc.create_client(
	config_file=basepath + '/config.json',
	keys_file=basepath + '/keys_api.mendeley.com.pkl')
group = [_ for _ in mendeley.groups() if _['name'] == group_to_sync]

if group == []:
	output.print_error(ot.errors.GROUP_NOT_FOUND)
group_id = int(group[0]['id'])

# check cache
fh = open(basepath + '/groupcache.pkl', 'a+')
try:
	cache = pickle.load(fh)
except:
	cache = dict()

output.print_info('Checking for outdated cache entries.').add_level()
current = mendeley.group_documents(group_id, items=20000)
for doc in current['documents']:
	if doc['id'] in cache and cache[doc['id']]._timestamp == doc['version']:
		continue
	details = mendeley.group_doc_details(group_id, doc['id'])
	if doc['id'] in cache:
		cache['id']._timestamp = details['version']
		cache['id']._citation = details['citation_key']
		output.print_info('Updated entry for %s' % details['citation_key'])
	else:
		try:
			doi = details['doi']
		except:
			doi = None
		p = paper(details['id'], details['version'], details['citation_key'], doi)
		cache[doc['id']] = p
		output.print_info('Added entry for %s' % details['citation_key'])
pickle.dump(cache, fh)
output.del_level()

# search key
output.print_info('Gathering information for missing keys.').add_level()
for citation in diff:
	output.print_info('Searching for %s' % citation).add_level()
	candidates = [_ for _ in cache.itervalues() if _._citation == citation]
	if len(candidates) > 1:
		output.print_warn(ot.errors.AMBIGUOUS_KEY).del_level()
		continue
	if len(candidates) == 0:
		output.print_warn(ot.errors.UNKNOWN_KEY).del_level()
		continue
	details = mendeley.group_doc_details(group_id, candidates[0]._id)
	warnings = False
	try:
		doi = details['doi']
	except:
		output.print_warn(ot.errors.NO_DOI)
		warnings = True
	try:
		journal = details['published_in']
	except:
		output.print_warn(ot.errors.NO_JOURNAL)
		warnings = True
	try:
		number = details['issue']
	except:
		output.print_warn(ot.errors.NO_ISSUE)
		warnings = True
	try:
		pages = details['pages']
	except:
		output.print_warn(ot.errors.NO_PAGES)
		warnings = True
	try:
		volume = details['volume']
	except:
		output.print_warn(ot.errors.NO_VOLUME)
		warnings = True
	try:
		year = details['year']
	except:
		output.print_warn(ot.errors.NO_YEAR)
		warnings = True
	try:
		authors = details['authors']
	except:
		output.print_warn(ot.errors.NO_AUTHORS)
		warnings = True

	if warnings:
		output.print_info('Omitting entry due to warnings.').del_level()
		continue

	authors = ' and '.join([_['surname'] + ', ' + _['forename'] for _ in details['authors']])
	entry = u'''
@article{%s,
author = {%s},
doi = {%s},
journal = {%s},
number = {%s},
pages = {%s},
volume = {%s},
year = {%s}
}
''' % (citation, authors, doi, journal, number, pages, volume, year)
	entry = bibtex_escape(entry)
	fhbib.seek(0, 2)
	fhbib.write(entry)
	output.print_info('Added entry to bibfile.').del_level()

fhbib.flush()
output.del_level()

# check for identical citation keys for identical DOI
output.print_info('Checking citation keys for consistency.').add_level()
dois_bib = dict()
fhbib.seek(0)
lines = [_.strip() for _ in fhbib.readlines() if _.strip().lower().startswith('@article') or _.strip().startswith('doi')]
article = None
for line in lines:
	if line.lower().startswith('@article'):
		if article is not None:
			output.print_info('For entry %s in local bibfile:' % article).add_level()
			output.print_warn(ot.errors.NO_DOI).del_level()
		article = re.findall(r'@article{([^,]+),', line, re.IGNORECASE)[0]
	else:
		doi = re.findall(r'doi.*{([^,]+)}', line, re.IGNORECASE)
		if len(doi) == 1:
			dois_bib[doi[0]] = article
			article = None

dois_mendeley = dict()
for entry in cache.itervalues():
	if entry._doi is not None:
		dois_mendeley[entry._doi] = entry._citation

for doi, citation in dois_bib.iteritems():
	if doi in dois_mendeley:
		if dois_mendeley[doi] != citation:
			output.print_info('You should use %s instead of %s.' % (dois_mendeley[doi], citation))
	else:
		output.print_info('You should add %s (%s) to the Mendeley database.' % (citation, doi))
