#!/usr/bin/env python
import sys 

class errors(object):
	NO_MODE_AND_INPUT 	= (1, 'Mode and input file missing.')
	NO_OUTPUT 			= (2, 'No output file given.')
	OUTPUT_PRESENT 		= (3, 'Output file already present.')
	NO_Z_MATRIX 		= (4, 'No Z-matrix found.')
	UNEXPECTED_EOF 		= (5, 'Unexpected end of file.')
	INCOMPLETE_COORD 	= (6, 'Incomplete coordinate set.')
	PARSE_ERROR 		= (7, 'Parse error(s).')
	NO_CONFORMATIONS	= (8, 'No conformations found.')
	UNKNOWN_MODE        = (9, 'Unknown operation mode.')
	MULTIPLE_KEYPRESS	= (10, 'Please stick to one key press at a time.')
	NO_SAMPLES			= (11, 'No samples to work with.')
	BEAT_MISMATCH 		= (12, 'Discarding samples due to beat count mismatch.')
	SAMPLE_NONEMPTY		= (13, 'Sample already recorded.')
	ONE_SAMPLE			= (14, 'Only one sample left.')
	PREMATURE_KEYPRESS  = (15, 'First keypress before initialization was complete.')
	SAMPLE_EMPTY		= (16, 'Sample contains no beats.')
	USE_CACHED			= (17, 'Using cached values.')
	VIOLATED_CONSTRAINT = (18, 'Constraint violated.')
	CACHE_OUTDATED		= (19, 'Cached data is outdated.')
	INTERNAL_ERROR		= (20, 'Internal error.')
	UNSUPPORTED_FORMAT	= (21, 'Unsupported format.')
	POSSIBLE_ISSUE 		= (22, 'Possible issue. Please check.')
	MISSING_FRAMES		= (23, 'Unknown parameter frames.')
	MISSING_ATOMS		= (24, 'Unknown parameter frames.')
	NOT_IMPLEMENTED		= (25, 'Feature not implemented.')
	EMPTY_ATOM_LABEL	= (26, 'Empty atom label.')
	NO_PARAMETER_FILES	= (27, 'No force-field parameter files found.')
	CGENFF_OMITTED		= (28, 'Data from CGenFF found but omitted.')
	FILE_IGNORED		= (29, 'Ignoring file.')
	CHARMM_OMITTED		= (30, 'CHARMM parameter file omitted.')
	UNKNOWN_REQUEST		= (31, 'Read unknown request.')
	NO_D_MATRIX 		= (32, 'No distance matrix found.')
	NO_RESIDUE	 		= (33, 'At least one residue has to be specified.')
	TOO_MANY_RESIDUES	= (34, 'Only one residue can be analyzed at a time.')
	NO_REFERENCE_DATA	= (35, 'No reference data found.')
	NO_SCF_DATA			= (36, 'No SCF energies found.')
	NOT_CONVERGED		= (37, 'Geometric optimization did not converge.')
	GROUP_NOT_FOUND		= (38, 'Unable to select group.')
	AMBIGUOUS_KEY		= (39, 'Found multiple publications with the same citation key.')
	UNKNOWN_KEY			= (40, 'Found no publication with this citation key.')
	NO_DOI				= (41, 'No DOI entry found.')
	NO_JOURNAL			= (42, 'No journal entry found.')
	NO_ISSUE			= (43, 'No issue entry found.')
	NO_PAGES			= (44, 'No pages entry found.')
	NO_VOLUME			= (45, 'No volume entry found.')
	NO_YEAR				= (46, 'No year entry found.')
	NO_AUTHORS			= (47, 'No authors entry found.')

# sanity check for error codes
codes = [x[1][0] for x in vars(errors).items() if x[0].upper() == x[0]]
if len(codes) != len(set(codes)):
	raise TypeError('Duplicate error code.')

class null_output(object):
	def __getattribute__(self, name):
		return lambda: self

class output_tracker(object):
	def __enter__(self):
		self._stdout = sys.stdout
		sys.stdout = self

	def __exit__(self, exc_type, exc_value, traceback):
		sys.stdout = self._stdout

	def __init__(self):
		self._indent = 0
		self._indent_steps = 3
		self._stdout = sys.stdout

		# bash color codes
		self.colors = dict([
			('red', '0;31'), 
			('green', '0;32'),
			('brown', '0;33'),
			('default', '0')
			])

	def write(self, text):
		self.print_info(text.strip('\n'))

	def flush(self):
		pass

	def _print_multiline(self, prefix, message, color):
		if len(prefix) not in  [0, 2]:
			raise ValueError('Internal Error: Wrong prefix length.')
		if prefix != '':
			prefix += ' '
		if type(message) != str:
			message = str(message)

		parts = message.split('\n')
		target = self._stdout
		if not target.isatty():
			target = sys.stderr
		target.write('{1}\033[{3}m{0}\033[0m{2}\n'.format(
			prefix.upper(), 
			' ' * self._indent, 
			parts[0], 
			color
			))
		for part in parts[1:]:
			target.write('{0}{1}\n'.format(' ' * (self._indent + 2), part))

	def print_error(self, code):
		if not type(code[0]) is int:
			raise ValueError('Error code is not numeric.')
		self._print_multiline(
			'EE', 
			code[1] + (' (error code %d)' % code[0]), 
			self.colors['red']
			)
		exit(1)

	def print_warn(self, code):
		if not type(code[0]) is int:
			raise ValueError('Warning code is not numeric.')
		self._print_multiline(
			'WW', 
			code[1] + (' (warning code %d)' % code[0]), 
			self.colors['brown']
			)
		return self

	def print_info(self, message):
		self._print_multiline('II', message, self.colors['green'])
		return self

	def add_level(self):
		self._indent += self._indent_steps
		return self

	def del_level(self, depth=1):
		self._indent -= self._indent_steps*depth
		if self._indent < 0:
			raise ValueError('Leftmost indentation level reached.')
		return self

	def print_plain(self, message):
		self._print_multiline('', message, self.colors['default'])
		return self
